@@ -36,7 +36,7 @@
  * to the project. For more information see the website or contact
  * the copyright holders.
  *
- * $Id: net.c,v 1.26 2005/08/28 19:30:29 kattemat Exp $
+ * $Id: net.c,v 1.36 2007/05/02 07:41:20 bernd67 Exp $
  */
 
 #include "defs.h"
@@ -58,11 +58,16 @@
 #include <netinet/ip_icmp.h>
 #include <netinet/icmp_var.h>
 #include <netinet/icmp6.h>
+#include <netinet6/in6_var.h> /* For struct in6_ifreq */
+#include <ifaddrs.h>
+#include <sys/uio.h>
 #endif
 
 #ifdef __FreeBSD__
+#include <ifaddrs.h>
 #include <net/if_var.h>
 #include <net/ethernet.h>
+#include <netinet/in_var.h>
 #ifndef FBSD_NO_80211
 #include <net80211/ieee80211.h>
 #include <net80211/ieee80211_ioctl.h>
@@ -71,8 +76,15 @@
 #endif
 #endif
 
-#ifdef SPOOF
+#ifdef __MacOSX__
+#include <ifaddrs.h>
+#include <net/if_var.h>
+#include <net/ethernet.h>
+#include <netinet/in_var.h>
+#endif
+
 #include <net/if_dl.h>
+#ifdef SPOOF
 #include <libnet.h>
 #endif /* SPOOF */
 
@@ -88,7 +100,11 @@ static int gateway;
 static int set_sysctl_int(char *name, int new)
 {
   int old;
+#if __MacOSX__ || __OpenBSD__
+  size_t len = sizeof (old);
+#else
   unsigned int len = sizeof (old);
+#endif
 
 #ifdef __OpenBSD__
   int mib[4];
@@ -172,6 +188,17 @@ disable_redirects_global(int version)
     name = "net.inet6.icmp6.rediraccept";
 
   ignore_redir = set_sysctl_int(name, 0);
+#elif defined __FreeBSD__ || defined __MacOSX__
+  if (olsr_cnf->ip_version == AF_INET)
+  {
+    name = "net.inet.icmp.drop_redirect";
+    ignore_redir = set_sysctl_int(name, 1);
+  }
+  else
+  {
+    name = "net.inet6.icmp6.rediraccept";
+    ignore_redir = set_sysctl_int(name, 0);
+  }
 #else
   if (olsr_cnf->ip_version == AF_INET)
     name = "net.inet.icmp.drop_redirect";
@@ -207,7 +234,7 @@ disable_redirects_global(int version)
   return 1;
 }
 
-int disable_redirects(char *if_name, int index, int version)
+int disable_redirects(const char *if_name, struct interface *iface, int version)
 {
   // this function gets called for each interface olsrd uses; however,
   // FreeBSD can only globally control ICMP redirects, and not on a
@@ -216,7 +243,7 @@ int disable_redirects(char *if_name, int index, int version)
   return 1;
 }
 
-int deactivate_spoof(char *if_name, int index, int version)
+int deactivate_spoof(const char *if_name, struct interface *iface, int version)
 {
   return 1;
 }
@@ -243,6 +270,12 @@ int restore_settings(int version)
   else
     name = "net.inet6.icmp6.rediraccept";
 
+#elif defined __FreeBSD__ || defined __MacOSX__
+  if (olsr_cnf->ip_version == AF_INET)
+    name = "net.inet.icmp.drop_redirect";
+
+  else
+    name = "net.inet6.icmp6.rediraccept";
 #else
   if (olsr_cnf->ip_version == AF_INET)
     name = "net.inet.icmp.drop_redirect";
@@ -290,6 +323,7 @@ gethemusocket(struct sockaddr_in *pin)
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) 
     {
       perror("SO_REUSEADDR failed");
+      close(sock);
       return (-1);
     }
   /* connect to PORT on HOST */
@@ -298,6 +332,7 @@ gethemusocket(struct sockaddr_in *pin)
       printf("FAILED\n");
       fprintf(stderr, "Error connecting %d - %s\n", errno, strerror(errno));
       printf("connection refused\n");
+      close(sock);
       return (-1);
     }
 
@@ -332,22 +367,23 @@ getsocket(struct sockaddr *sa, int bufspace, char *int_name)
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) 
     {
       perror("SO_REUSEADDR failed");
+      close(sock);
       return (-1);
     }
 
-#ifdef SPOOF
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, &on, sizeof(on)) < 0) 
     {
       perror("SO_REUSEPORT failed");
+      close(sock);
       return (-1);
     }
 
   if (setsockopt(sock, IPPROTO_IP, IP_RECVIF, &on, sizeof(on)) < 0) 
     {
       perror("IP_RECVIF failed");
+      close(sock);
       return (-1);
     }
-#endif /* SPOOF */
 
   for (on = bufspace; ; on -= 1024) 
     {
@@ -403,9 +439,30 @@ int getsocket6(struct sockaddr_in6 *sin, int bufspace, char *int_name)
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) 
     {
       perror("SO_REUSEADDR failed");
+      close(sock);
       return (-1);
     }
 
+  if (setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, &on, sizeof(on)) < 0) 
+    {
+      perror("SO_REUSEPORT failed");
+      return (-1);
+    }
+
+#ifdef IPV6_RECVPKTINFO
+  if (setsockopt(sock, IPPROTO_IPV6, IPV6_RECVPKTINFO, &on, sizeof(on)) < 0)
+    {
+      perror("IPV6_RECVPKTINFO failed");
+      return (-1);
+    }
+#elif defined IPV6_PKTINFO
+    if (setsockopt(sock, IPPROTO_IPV6, IPV6_PKTINFO, &on, sizeof(on)) < 0)
+      {
+	perror("IPV6_PKTINFO failed");
+	return (-1);
+      }
+#endif
+
   if (bind(sock, (struct sockaddr *)sin, sizeof (*sin)) < 0) 
     {
       perror("bind");
@@ -425,31 +482,16 @@ int getsocket6(struct sockaddr_in6 *sin, int bufspace, char *int_name)
 int
 join_mcast(struct interface *ifs, int sock)
 {
-  /* See linux/in6.h */
+  /* See netinet6/in6.h */
 
   struct ipv6_mreq mcastreq;
 
   COPY_IP(&mcastreq.ipv6mr_multiaddr, &ifs->int6_multaddr.sin6_addr);
   mcastreq.ipv6mr_interface = ifs->if_index;
 
-#if 0
-  OLSR_PRINTF(3, "Interface %s joining multicast %s...",	ifs->int_name, olsr_ip_to_string((union olsr_ip_addr *)&ifs->int6_multaddr.sin6_addr))
-  /* Send multicast */
-  if(setsockopt(sock, 
-		IPPROTO_IPV6, 
-		IPV6_ADD_MEMBERSHIP, 
-		(char *)&mcastreq, 
-		sizeof(struct ipv6_mreq)) 
-     < 0)
-    {
-      perror("Join multicast");
-      return -1;
-    }
-#else
-#warning implement IPV6_ADD_MEMBERSHIP
-#endif
+  OLSR_PRINTF(3, "Interface %s joining multicast %s...",	ifs->int_name, olsr_ip_to_string((union olsr_ip_addr *)&ifs->int6_multaddr.sin6_addr));
 
-  /* Old libc fix */
+  /* rfc 3493 */
 #ifdef IPV6_JOIN_GROUP
   /* Join reciever group */
   if(setsockopt(sock, 
@@ -458,8 +500,8 @@ join_mcast(struct interface *ifs, int sock)
 		(char *)&mcastreq, 
 		sizeof(struct ipv6_mreq)) 
      < 0)
-#else
-  /* Join reciever group */
+#else /* rfc 2133, obsoleted */
+  /* Join receiver group */
   if(setsockopt(sock, 
 		IPPROTO_IPV6, 
 		IPV6_ADD_MEMBERSHIP, 
@@ -485,7 +527,7 @@ join_mcast(struct interface *ifs, int sock)
     }
 
 
-  OLSR_PRINTF(3, "OK\n")
+  OLSR_PRINTF(3, "OK\n");
   return 0;
 }
 
@@ -494,6 +536,70 @@ join_mcast(struct interface *ifs, int sock)
 
 int get_ipv6_address(char *ifname, struct sockaddr_in6 *saddr6, int scope_in)
 {
+  struct ifaddrs *ifap, *ifa;
+  const struct sockaddr_in6 *sin6 = NULL;
+  struct in6_ifreq ifr6;
+  int found = 0;
+  int s6;
+  u_int32_t flags6;
+
+  if (getifaddrs(&ifap) != 0)
+    {
+      OLSR_PRINTF(3, "get_ipv6_address: getifaddrs() failed.\n");
+      return 0;
+    }
+
+  for (ifa = ifap; ifa; ifa = ifa->ifa_next)
+    {
+      if (ifa->ifa_addr->sa_family == AF_INET6 &&
+          strcmp(ifa->ifa_name, ifname) == 0)
+        {
+	  sin6 = (const struct sockaddr_in6 *)ifa->ifa_addr;
+	  if (IN6_IS_ADDR_LINKLOCAL(&sin6->sin6_addr))
+	    continue;
+	  strncpy(ifr6.ifr_name, ifname, sizeof(ifname));
+	  if ((s6 = socket(AF_INET6, SOCK_DGRAM, 0)) < 0)
+	    {
+	      OLSR_PRINTF(3, "socket(AF_INET6,SOCK_DGRAM)");
+	      break;
+	    }
+	  ifr6.ifr_addr = *sin6;
+	  if (ioctl(s6, SIOCGIFAFLAG_IN6, &ifr6) < 0)
+	    {
+	      OLSR_PRINTF(3, "ioctl(SIOCGIFAFLAG_IN6)");
+	      close(s6);
+	      break;
+	    }
+	  close(s6);
+	  flags6 = ifr6.ifr_ifru.ifru_flags6;
+	  if ((flags6 & IN6_IFF_ANYCAST) != 0)
+	    continue;
+	  if (IN6_IS_ADDR_SITELOCAL(&sin6->sin6_addr))
+	    {
+	      if (scope_in)
+		{
+		  memcpy(&saddr6->sin6_addr, &sin6->sin6_addr,
+			 sizeof(struct in6_addr));
+		  found = 1;
+		  break;
+		}
+	    }
+	  else
+	    {
+	      if (scope_in == 0)
+		{
+		  memcpy(&saddr6->sin6_addr, &sin6->sin6_addr,
+			 sizeof(struct in6_addr));
+		  found = 1;
+		  break;
+		}
+	    }
+	}
+    }
+  freeifaddrs(ifap);
+  if (found)
+    return 1;
+
   return 0;
 }
 
@@ -536,7 +642,7 @@ olsr_sendto(int s,
   context = libnet_init(LIBNET_LINK, iface->int_name, errbuf);
   if (context == NULL)
     {
-      OLSR_PRINTF (1, "libnet init: %s\n", libnet_geterror (context))
+      OLSR_PRINTF (1, "libnet init: %s\n", libnet_geterror (context));
       return (0);
     }
 
@@ -556,7 +662,7 @@ olsr_sendto(int s,
 			      udp_tag);				/* pblock */
   if (udp_tag == -1)
     {
-      OLSR_PRINTF (1, "libnet UDP header: %s\n", libnet_geterror (context))
+      OLSR_PRINTF (1, "libnet UDP header: %s\n", libnet_geterror (context));
 	return (0);
     }
 
@@ -575,7 +681,7 @@ olsr_sendto(int s,
 			      ip_tag);				/* pblock */
   if (ip_tag == -1)
     {
-      OLSR_PRINTF (1, "libnet IP header: %s\n", libnet_geterror (context))
+      OLSR_PRINTF (1, "libnet IP header: %s\n", libnet_geterror (context));
       return (0);
     }
 
@@ -588,14 +694,14 @@ olsr_sendto(int s,
 				     ether_tag);  		/* pblock tag */
   if (ether_tag == -1)
     {
-      OLSR_PRINTF (1, "libnet ethernet header: %s\n", libnet_geterror (context))
+      OLSR_PRINTF (1, "libnet ethernet header: %s\n", libnet_geterror (context));
       return (0);
     }
  
   status = libnet_write (context);
   if (status == -1)
     {
-      OLSR_PRINTF (1, "libnet packet write: %s\n", libnet_geterror (context))
+      OLSR_PRINTF (1, "libnet packet write: %s\n", libnet_geterror (context));
       return (0);
     }
 
@@ -621,16 +727,19 @@ olsr_recvfrom(int  s,
 	      struct sockaddr *from,
 	      socklen_t *fromlen)
 {
-#if SPOOF
   struct msghdr mhdr;
   struct iovec iov;
   struct cmsghdr *cm;
   struct sockaddr_dl *sdl;
   struct sockaddr_in *sin = (struct sockaddr_in *) from; //XXX
+  struct sockaddr_in6 *sin6;
+  struct in6_addr *iaddr6;
+  struct in6_pktinfo *pkti;
+  struct interface *ifc;
+  char addrstr[INET6_ADDRSTRLEN];
+  char iname[IFNAMSIZ];
   unsigned char chdr[4096];
   int count;
-  struct interface *ifc;
-  char iname[32];
 
   bzero(&mhdr, sizeof(mhdr));
   bzero(&iov, sizeof(iov));
@@ -653,35 +762,45 @@ olsr_recvfrom(int  s,
 
   /* this needs to get communicated back to caller */
   *fromlen = mhdr.msg_namelen;
-
-  cm = (struct cmsghdr *) chdr;
-  sdl = (struct sockaddr_dl *) CMSG_DATA (cm);
-  bzero (iname, sizeof (iname));
-  memcpy (iname, sdl->sdl_data, sdl->sdl_nlen);
+  if (olsr_cnf->ip_version == AF_INET6)
+    {
+      for (cm = (struct cmsghdr *)CMSG_FIRSTHDR(&mhdr); cm;
+	   cm = (struct cmsghdr *)CMSG_NXTHDR(&mhdr, cm))
+	{
+	  if (cm->cmsg_level == IPPROTO_IPV6 && cm->cmsg_type == IPV6_PKTINFO)
+	    {
+	      pkti = (struct in6_pktinfo *) CMSG_DATA(cm);
+	      iaddr6 = &pkti->ipi6_addr;
+	      if_indextoname(pkti->ipi6_ifindex, iname);
+	    }
+	}
+    }
+  else
+    {
+      cm = (struct cmsghdr *) chdr;
+      sdl = (struct sockaddr_dl *) CMSG_DATA (cm);
+      bzero (iname, sizeof (iname));
+      memcpy (iname, sdl->sdl_data, sdl->sdl_nlen);
+    }
 
   ifc = if_ifwithsock (s);
 
+  sin6 = (struct sockaddr_in6 *)from;
+  OLSR_PRINTF (4, "%d bytes from %s, socket associated %s really received on %s\n",
+	       count,
+	       (olsr_cnf->ip_version == AF_INET6) ?
+		 inet_ntop(AF_INET6, (char *)&sin6->sin6_addr, addrstr,
+				     INET6_ADDRSTRLEN):
+	         inet_ntoa (sin->sin_addr),
+	       ifc->int_name,
+	       iname);
+
   if (strcmp (ifc->int_name, iname) != 0)
     {
       return (0);
     }
 
-  OLSR_PRINTF (2, "%d bytes from %s, socket associated %s really received on %s\n",
-	       count,
-	       inet_ntoa (sin->sin_addr),
-	       ifc->int_name,
-	       iname);
-
   return (count);
-
-#else /* SPOOF */
-  return recvfrom(s, 
-		  buf, 
-		  len, 
-		  0, 
-		  from, 
-		  fromlen);
-#endif /* SPOOF */
 }
 
 /**
@@ -719,7 +838,7 @@ check_wireless_interface(char *ifname)
   strlcpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
   ifr.ifr_data = (caddr_t)&wreq;
 
-  return (ioctl(ioctl_s, SIOCGWAVELAN, &ifr) >= 0) ? 1 : 0;
+  return (ioctl(olsr_cnf->ioctl_s, SIOCGWAVELAN, &ifr) >= 0) ? 1 : 0;
 #else
   return 0;
 #endif
@@ -745,13 +864,13 @@ calculate_if_metric(char *ifname)
       memset(&ifm, 0, sizeof(ifm));
       strlcpy(ifm.ifm_name, ifname, sizeof(ifm.ifm_name));
 
-      if(ioctl(ioctl_s, SIOCGIFMEDIA, &ifm) < 0)
+      if(ioctl(olsr_cnf->ioctl_s, SIOCGIFMEDIA, &ifm) < 0)
 	{
-	  OLSR_PRINTF(1, "Error SIOCGIFMEDIA(%s)\n", ifm.ifm_name)
+	  OLSR_PRINTF(1, "Error SIOCGIFMEDIA(%s)\n", ifm.ifm_name);
 	  return WEIGHT_ETHERNET_DEFAULT;
 	}
 
-      OLSR_PRINTF(1, "%s: STATUS 0x%08x\n", ifm.ifm_name, ifm.ifm_status)
+      OLSR_PRINTF(1, "%s: STATUS 0x%08x\n", ifm.ifm_name, ifm.ifm_status);
 #endif
       return WEIGHT_ETHERNET_DEFAULT;
     }
