@@ -36,7 +36,7 @@
  * to the project. For more information see the website or contact
  * the copyright holders.
  *
- * $Id: net.c,v 1.29 2005/09/17 20:48:50 kattemat Exp $
+ * $Id: net.c,v 1.34 2007/04/25 22:08:18 bernd67 Exp $
  */
 
 
@@ -57,6 +57,10 @@
 #define SIOCGIWNAME	0x8B01		/* get name == wireless protocol */
 #define SIOCGIWRATE	0x8B21		/* get default bit rate (bps) */
 
+/* The original state of the IP forwarding proc entry */
+static char orig_fwd_state;
+static char orig_global_redirect_state;
+
 /**
  *Bind a socket to a device
  *
@@ -72,14 +76,11 @@ bind_socket_to_device(int sock, char *dev_name)
   /*
    *Bind to device using the SO_BINDTODEVICE flag
    */
-  OLSR_PRINTF(3, "Binding socket %d to device %s\n", sock, dev_name)
+  OLSR_PRINTF(3, "Binding socket %d to device %s\n", sock, dev_name);
   return setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE, dev_name, strlen(dev_name)+1);
-
 }
 
 
-
-
 /**
  *Enable IP forwarding.
  *Just writing "1" to the /proc/sys/net/ipv4/ip_forward
@@ -96,20 +97,9 @@ int
 enable_ip_forwarding(int version)
 {
   FILE *proc_fwd;
-  char procfile[FILENAME_MAX];
-
-  if(version == AF_INET)
-    {
-      strncpy(procfile, "/proc/sys/net/ipv4/ip_forward", FILENAME_MAX);
-    }
-  else
-    if(version == AF_INET6)
-      {
-	strncpy(procfile, "/proc/sys/net/ipv6/conf/all/forwarding", FILENAME_MAX);
-      }
-    else
-      return -1;
-
+  const char * const procfile = version == AF_INET
+      ? "/proc/sys/net/ipv4/ip_forward"
+      : "/proc/sys/net/ipv6/conf/all/forwarding";
 
   if ((proc_fwd=fopen(procfile, "r"))==NULL)
     {
@@ -124,48 +114,40 @@ enable_ip_forwarding(int version)
       return 0;
     }
   
+  orig_fwd_state = fgetc(proc_fwd);
+  fclose(proc_fwd);
+  if(orig_fwd_state == '1')
+    {
+      OLSR_PRINTF(3, "\nIP forwarding is enabled on this system\n");
+    }
   else
     {
-      orig_fwd_state = fgetc(proc_fwd);
-      fclose(proc_fwd);
-      if(orig_fwd_state == '1')
-	{
-	  OLSR_PRINTF(3, "\nIP forwarding is enabled on this system\n")
-	}
+      if ((proc_fwd=fopen(procfile, "w"))==NULL)
+        {
+          fprintf(stderr, "Could not open %s for writing!\n", procfile);
+          fprintf(stderr, "I will continue(in 3 sec) - but you should mannually ensure that IP forwarding is enabeled!\n\n");
+          sleep(3);
+          return 0;
+        }
       else
-	{
-	  if ((proc_fwd=fopen(procfile, "w"))==NULL)
-	    {
-	      fprintf(stderr, "Could not open %s for writing!\n", procfile);
-	      fprintf(stderr, "I will continue(in 3 sec) - but you should mannually ensure that IP forwarding is enabeled!\n\n");
-	      sleep(3);
-	      return 0;
-	    }
-	  else
-	    {
-	      syslog(LOG_INFO, "Writing \"1\" to %s\n", procfile);
-	      fputs("1", proc_fwd);
-	    }
-	  fclose(proc_fwd);
-
-	}
+        {
+          syslog(LOG_INFO, "Writing \"1\" to %s\n", procfile);
+          fputs("1", proc_fwd);
+        }
+      fclose(proc_fwd);
     }
-  return 1;
-      
+  return 1;      
 }
 
-
 int
 disable_redirects_global(int version)
 {
   FILE *proc_redirect;
-  char procfile[FILENAME_MAX];
+  const char * const procfile = "/proc/sys/net/ipv4/conf/all/send_redirects";
 
   if(version == AF_INET6)
     return -1;
 
-  strcpy(procfile, "/proc/sys/net/ipv4/conf/all/send_redirects");
-
   if((proc_redirect = fopen(procfile, "r")) == NULL)
     {
       fprintf(stderr, "WARNING! Could not open the %s file to check/disable ICMP redirects!\nAre you using the procfile filesystem?\nDoes your system support IPv4?\nI will continue(in 3 sec) - but you should mannually ensure that ICMP redirects are disabled!\n\n", procfile);
@@ -173,16 +155,11 @@ disable_redirects_global(int version)
       sleep(3);
       return -1;
     }
-  else
-    {
-      orig_global_redirect_state = fgetc(proc_redirect);
-      fclose(proc_redirect);
-    }
+  orig_global_redirect_state = fgetc(proc_redirect);
+  fclose(proc_redirect);
 
   if(orig_global_redirect_state == '0')
-    {
       return 0;
-    }
 
   if ((proc_redirect = fopen(procfile, "w"))==NULL)
     {
@@ -191,23 +168,18 @@ disable_redirects_global(int version)
       sleep(3);
       return 0;
     }
-  else
-    {
-      syslog(LOG_INFO, "Writing \"0\" to %s", procfile);
-      fputs("0", proc_redirect);
-    }
-  fclose(proc_redirect);
-  
+  syslog(LOG_INFO, "Writing \"0\" to %s", procfile);
+  fputs("0", proc_redirect);
+  fclose(proc_redirect);  
   return 1;
 }
 
-
 /**
  *
  *@return 1 on sucess 0 on failiure
  */ 
 int
-disable_redirects(char *if_name, int index, int version)
+disable_redirects(const char *if_name, struct interface *iface, int version)
 {
   FILE *proc_redirect;
   char procfile[FILENAME_MAX];
@@ -216,22 +188,17 @@ disable_redirects(char *if_name, int index, int version)
     return -1;
 
   /* Generate the procfile name */
-  sprintf(procfile, REDIRECT_PROC, if_name);
+  snprintf(procfile, sizeof(procfile), REDIRECT_PROC, if_name);
 
 
   if((proc_redirect = fopen(procfile, "r")) == NULL)
     {
-      fprintf(stderr, "WARNING! Could not open the %s file to check/disable ICMP redirects!\nAre you using the procfile filesystem?\nDoes your system support IPv4?\nI will continue(in 3 sec) - but you should mannually ensure that ICMP redirects are disabled!\n\n", procfile);
-      
+      fprintf(stderr, "WARNING! Could not open the %s file to check/disable ICMP redirects!\nAre you using the procfile filesystem?\nDoes your system support IPv4?\nI will continue(in 3 sec) - but you should mannually ensure that ICMP redirects are disabled!\n\n", procfile);      
       sleep(3);
       return 0;
     }
-  else
-    {
-      nic_states[index].redirect = fgetc(proc_redirect);
-      fclose(proc_redirect);
-      
-    }
+  iface->nic_state.redirect = fgetc(proc_redirect);
+  fclose(proc_redirect);      
 
   if ((proc_redirect = fopen(procfile, "w"))==NULL)
     {
@@ -240,24 +207,18 @@ disable_redirects(char *if_name, int index, int version)
       sleep(3);
       return 0;
     }
-  else
-    {
-      syslog(LOG_INFO, "Writing \"0\" to %s", procfile);
-      fputs("0", proc_redirect);
-    }
+  syslog(LOG_INFO, "Writing \"0\" to %s", procfile);
+  fputs("0", proc_redirect);
   fclose(proc_redirect);
-
   return 1;
 }
 
-
-
 /**
  *
  *@return 1 on sucess 0 on failiure
  */ 
 int
-deactivate_spoof(char *if_name, int index, int version)
+deactivate_spoof(const char *if_name, struct interface *iface, int version)
 {
   FILE *proc_spoof;
   char procfile[FILENAME_MAX];
@@ -265,11 +226,9 @@ deactivate_spoof(char *if_name, int index, int version)
   if(version == AF_INET6)
     return -1;
 
-
   /* Generate the procfile name */
   sprintf(procfile, SPOOF_PROC, if_name);
 
-
   if((proc_spoof = fopen(procfile, "r")) == NULL)
     {
       fprintf(stderr, "WARNING! Could not open the %s file to check/disable the IP spoof filter!\nAre you using the procfile filesystem?\nDoes your system support IPv4?\nI will continue(in 3 sec) - but you should mannually ensure that IP spoof filtering is disabled!\n\n", procfile);
@@ -277,12 +236,8 @@ deactivate_spoof(char *if_name, int index, int version)
       sleep(3);
       return 0;
     }
-  else
-    {
-      nic_states[index].spoof = fgetc(proc_spoof);
-      fclose(proc_spoof);
-      
-    }
+  iface->nic_state.spoof = fgetc(proc_spoof);
+  fclose(proc_spoof);
 
   if ((proc_spoof = fopen(procfile, "w")) == NULL)
     {
@@ -291,42 +246,29 @@ deactivate_spoof(char *if_name, int index, int version)
       sleep(3);
       return 0;
     }
-  else
-    {
-      syslog(LOG_INFO, "Writing \"0\" to %s", procfile);
-      fputs("0", proc_spoof);
-    }
+  syslog(LOG_INFO, "Writing \"0\" to %s", procfile);
+  fputs("0", proc_spoof);
   fclose(proc_spoof);
-
   return 1;
 }
 
-
-
 /**
  *Resets the spoof filter and ICMP redirect settings
  */
-
 int
 restore_settings(int version)
 {
-  FILE *proc_fd;
-  char procfile[FILENAME_MAX];
   struct interface *ifs;
 
-  OLSR_PRINTF(1, "Restoring network state\n")
+  OLSR_PRINTF(1, "Restoring network state\n");
 
   /* Restore IP forwarding to "off" */
   if(orig_fwd_state == '0')
     {
-      if(version == AF_INET)
-	{
-	  strcpy(procfile, "/proc/sys/net/ipv4/ip_forward");
-	}
-      else if(version == AF_INET6)
-	{
-	  strcpy(procfile, "/proc/sys/net/ipv6/conf/all/forwarding");
-	}
+      const char * const procfile = version == AF_INET
+        ? "/proc/sys/net/ipv4/ip_forward"
+        : "/proc/sys/net/ipv6/conf/all/forwarding";
+      FILE *proc_fd;
 
       if ((proc_fd = fopen(procfile, "w")) == NULL)
 	{
@@ -338,7 +280,6 @@ restore_settings(int version)
 	  fputc(orig_fwd_state, proc_fd);
 	  fclose(proc_fd);
 	}
-
     }
 
   /* Restore global ICMP redirect setting */
@@ -346,7 +287,8 @@ restore_settings(int version)
     {
       if(version == AF_INET)
 	{
-	  strcpy(procfile, "/proc/sys/net/ipv4/conf/all/send_redirects");
+	  const char * const procfile = "/proc/sys/net/ipv4/conf/all/send_redirects";
+          FILE *proc_fd;
 
 	  if ((proc_fd = fopen(procfile, "w")) == NULL)
 	    {
@@ -361,56 +303,48 @@ restore_settings(int version)
 	}
     }
 
-
   if(version == AF_INET6)
     return 0;
 
   for(ifs = ifnet; ifs != NULL; ifs = ifs->int_next)
     {
+      char procfile[FILENAME_MAX];
+      FILE *proc_fd;
       /* Discard host-emulation interfaces */
       if(ifs->is_hcif)
 	continue;
       /* ICMP redirects */
       
       /* Generate the procfile name */
-      sprintf(procfile, REDIRECT_PROC, ifs->int_name);
+      snprintf(procfile, sizeof(procfile), REDIRECT_PROC, ifs->int_name);
       
       if ((proc_fd = fopen(procfile, "w")) == NULL)
-	{
 	  fprintf(stderr, "Could not open %s for writing!\nSettings not restored!\n", procfile);
-	}
       else
 	{
-	  syslog(LOG_INFO, "Resetting %s to %c\n", procfile, nic_states[ifs->if_nr].redirect);
+	  syslog(LOG_INFO, "Resetting %s to %c\n", procfile, ifs->nic_state.redirect);
 
-	  fputc(nic_states[ifs->if_nr].redirect, proc_fd);
+	  fputc(ifs->nic_state.redirect, proc_fd);
 	  fclose(proc_fd);
 	}
-
       
       /* Spoof filter */
       
       /* Generate the procfile name */
       sprintf(procfile, SPOOF_PROC, ifs->int_name);
-
       if ((proc_fd = fopen(procfile, "w")) == NULL)
-	{
 	  fprintf(stderr, "Could not open %s for writing!\nSettings not restored!\n", procfile);
-	}
       else
 	{
-	  syslog(LOG_INFO, "Resetting %s to %c\n", procfile, nic_states[ifs->if_nr].spoof);
+	  syslog(LOG_INFO, "Resetting %s to %c\n", procfile, ifs->nic_state.spoof);
 
-	  fputc(nic_states[ifs->if_nr].spoof, proc_fd);
+	  fputc(ifs->nic_state.spoof, proc_fd);
 	  fclose(proc_fd);
 	}
-
     }
   return 1;
-
 }
 
-
 /**
  *Creates a nonblocking broadcast socket.
  *@param sa sockaddr struct. Used for bind(2).
@@ -422,19 +356,18 @@ gethemusocket(struct sockaddr_in *pin)
   int sock, on = 1;
 
   OLSR_PRINTF(1, "       Connecting to switch daemon port 10150...");
-
-
   if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) 
     {
       perror("hcsocket");
       syslog(LOG_ERR, "hcsocket: %m");
-      return (-1);
+      return -1;
     }
 
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) 
     {
       perror("SO_REUSEADDR failed");
-      return (-1);
+      close(sock);
+      return -1;
     }
   /* connect to PORT on HOST */
   if (connect(sock,(struct sockaddr *) pin, sizeof(*pin)) < 0) 
@@ -442,13 +375,14 @@ gethemusocket(struct sockaddr_in *pin)
       printf("FAILED\n");
       fprintf(stderr, "Error connecting %d - %s\n", errno, strerror(errno));
       printf("connection refused\n");
-      return (-1);
+      close(sock);
+      return -1;
     }
 
   printf("OK\n");
 
   /* Keep TCP socket blocking */  
-  return (sock);
+  return sock;
 }
 
 
@@ -463,41 +397,34 @@ getsocket(struct sockaddr *sa, int bufspace, char *int_name)
   struct sockaddr_in *sin=(struct sockaddr_in *)sa;
   int sock, on = 1;
 
-
-
   if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) 
     {
       perror("socket");
       syslog(LOG_ERR, "socket: %m");
-      return (-1);
+      return -1;
     }
 
-
-
 #ifdef SO_BROADCAST
   if (setsockopt(sock, SOL_SOCKET, SO_BROADCAST, &on, sizeof (on)) < 0)
     {
       perror("setsockopt");
       syslog(LOG_ERR, "setsockopt SO_BROADCAST: %m");
       close(sock);
-      return (-1);
+      return -1;
     }
 #endif
 
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) 
     {
       perror("SO_REUSEADDR failed");
-      return (-1);
+      close(sock);
+      return -1;
     }
 
-
-
 #ifdef SO_RCVBUF
-
   for (on = bufspace; ; on -= 1024) 
     {
-      if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF,
-		     &on, sizeof (on)) == 0)
+      if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF, &on, sizeof (on)) == 0)
 	break;
       if (on <= 8*1024) 
 	{
@@ -506,11 +433,8 @@ getsocket(struct sockaddr *sa, int bufspace, char *int_name)
 	  break;
 	}
     }
-
-
 #endif
 
-
   /*
    * WHEN USING KERNEL 2.6 THIS MUST HAPPEN PRIOR TO THE PORT BINDING!!!!
    */
@@ -520,26 +444,25 @@ getsocket(struct sockaddr *sa, int bufspace, char *int_name)
     {
       fprintf(stderr, "Could not bind socket to device... exiting!\n\n");
       syslog(LOG_ERR, "Could not bind socket to device... exiting!\n\n");
+      close(sock);
       return -1;
     }
 
-
   if (bind(sock, (struct sockaddr *)sin, sizeof (*sin)) < 0) 
     {
       perror("bind");
       syslog(LOG_ERR, "bind: %m");
       close(sock);
-      return (-1);
+      return -1;
     }
-
   /*
-   *One should probably fetch the flags first
+   *FIXME: One should probably fetch the flags first
    *using F_GETFL....
    */
   if (fcntl(sock, F_SETFL, O_NONBLOCK) == -1)
     syslog(LOG_ERR, "fcntl O_NONBLOCK: %m\n");
 
-  return (sock);
+  return sock;
 }
 
 
@@ -552,9 +475,6 @@ int
 getsocket6(struct sockaddr_in6 *sin, int bufspace, char *int_name)
 {
   int sock, on = 1;
-
-
-
   if ((sock = socket(AF_INET6, SOCK_DGRAM, 0)) < 0) 
     {
       perror("socket");
@@ -583,14 +503,10 @@ getsocket6(struct sockaddr_in6 *sin, int bufspace, char *int_name)
   */
   //#endif
 
-
-
-
 #ifdef SO_RCVBUF
   for (on = bufspace; ; on -= 1024) 
     {
-      if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF,
-		     &on, sizeof (on)) == 0)
+      if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF, &on, sizeof (on)) == 0)
 	break;
       if (on <= 8*1024) 
 	{
@@ -599,17 +515,15 @@ getsocket6(struct sockaddr_in6 *sin, int bufspace, char *int_name)
 	  break;
 	}
     }
-
-
 #endif
 
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) 
     {
       perror("SO_REUSEADDR failed");
+      close(sock);
       return (-1);
     }
 
-
   /*
    * WHEN USING KERNEL 2.6 THIS MUST HAPPEN PRIOR TO THE PORT BINDING!!!!
    */
@@ -619,10 +533,10 @@ getsocket6(struct sockaddr_in6 *sin, int bufspace, char *int_name)
     {
       fprintf(stderr, "Could not bind socket to device... exiting!\n\n");
       syslog(LOG_ERR, "Could not bind socket to device... exiting!\n\n");
+      close(sock);
       return -1;
     }
 
-
   if (bind(sock, (struct sockaddr *)sin, sizeof (*sin)) < 0) 
     {
       perror("bind");
@@ -630,7 +544,6 @@ getsocket6(struct sockaddr_in6 *sin, int bufspace, char *int_name)
       close(sock);
       return (-1);
     }
-
   /*
    *One should probably fetch the flags first
    *using F_GETFL....
@@ -638,12 +551,9 @@ getsocket6(struct sockaddr_in6 *sin, int bufspace, char *int_name)
   if (fcntl(sock, F_SETFL, O_NONBLOCK) == -1)
     syslog(LOG_ERR, "fcntl O_NONBLOCK: %m\n");
 
-
-
-  return (sock);
+  return sock;
 }
 
-
 int
 join_mcast(struct interface *ifs, int sock)
 {
@@ -655,7 +565,7 @@ join_mcast(struct interface *ifs, int sock)
   mcastreq.ipv6mr_interface = ifs->if_index;
 
 #if !defined __FreeBSD__ && !defined __MacOSX__ && !defined __NetBSD__
-  OLSR_PRINTF(3, "Interface %s joining multicast %s...",	ifs->int_name, olsr_ip_to_string((union olsr_ip_addr *)&ifs->int6_multaddr.sin6_addr))
+  OLSR_PRINTF(3, "Interface %s joining multicast %s...",	ifs->int_name, olsr_ip_to_string((union olsr_ip_addr *)&ifs->int6_multaddr.sin6_addr));
   /* Send multicast */
   if(setsockopt(sock, 
 		IPPROTO_IPV6, 
@@ -692,9 +602,8 @@ join_mcast(struct interface *ifs, int sock)
     {
       perror("Join multicast send");
       return -1;
-    }
+    }  
 
-  
   if(setsockopt(sock, 
 		IPPROTO_IPV6, 
 		IPV6_MULTICAST_IF, 
@@ -707,17 +616,14 @@ join_mcast(struct interface *ifs, int sock)
     }
 
 
-  OLSR_PRINTF(3, "OK\n")
+  OLSR_PRINTF(3, "OK\n");
   return 0;
 }
 
-
-
 /*
  *From net-tools lib/interface.c
  *
  */
-
 int
 get_ipv6_address(char *ifname, struct sockaddr_in6 *saddr6, int scope_in)
 {
@@ -739,13 +645,13 @@ get_ipv6_address(char *ifname, struct sockaddr_in6 *saddr6, int scope_in)
 	      sprintf(addr6, "%s:%s:%s:%s:%s:%s:%s:%s",
 		      addr6p[0], addr6p[1], addr6p[2], addr6p[3],
 		      addr6p[4], addr6p[5], addr6p[6], addr6p[7]);
-	      OLSR_PRINTF(5, "\tinet6 addr: %s\n", addr6)
-	      OLSR_PRINTF(5, "\tScope: %d\n", scope)
+	      OLSR_PRINTF(5, "\tinet6 addr: %s\n", addr6);
+	      OLSR_PRINTF(5, "\tScope: %d\n", scope);
 	      if(scope == scope_in)
 		{
 		  OLSR_PRINTF(4, "Found addr: %s:%s:%s:%s:%s:%s:%s:%s\n",
 			      addr6p[0], addr6p[1], addr6p[2], addr6p[3],
-			      addr6p[4], addr6p[5], addr6p[6], addr6p[7])
+			      addr6p[4], addr6p[5], addr6p[6], addr6p[7]);
 		  inet_pton(AF_INET6,addr6,&tmp_sockaddr6);
 		  memcpy(&saddr6->sin6_addr, &tmp_sockaddr6, sizeof(struct in6_addr));	  
 		  fclose(f);
@@ -754,8 +660,7 @@ get_ipv6_address(char *ifname, struct sockaddr_in6 *saddr6, int scope_in)
 	    }
 	}
       fclose(f);
-    }
-  
+    }  
   return 0;
 }
 
@@ -763,7 +668,6 @@ get_ipv6_address(char *ifname, struct sockaddr_in6 *saddr6, int scope_in)
 /**
  * Wrapper for sendto(2)
  */
-
 ssize_t
 olsr_sendto(int s, 
 	    const void *buf, 
@@ -814,14 +718,14 @@ olsr_select(int nfds,
 }
 
 int
-check_wireless_interface(char *ifname)
+check_wireless_interface(char * ifname)
 {
   struct ifreq ifr;
 
   memset(&ifr, 0, sizeof(ifr));
   strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
 
-  return (ioctl(ioctl_s, SIOCGIWNAME, &ifr) >= 0) ? 1 : 0;
+  return (ioctl(olsr_cnf->ioctl_s, SIOCGIWNAME, &ifr) >= 0) ? 1 : 0;
 }
 
 #if 0
@@ -874,13 +778,13 @@ calculate_if_metric(char *ifname)
       strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
       
       /* Get bit rate */
-      if(ioctl(ioctl_s, SIOCGIWRATE, &ifr) < 0)
+      if(ioctl(olsr_cnf->ioctl_s, SIOCGIWRATE, &ifr) < 0)
 	{
-	  OLSR_PRINTF(1, "Not able to find rate for WLAN interface %s\n", ifname)
+          OLSR_PRINTF(1, "Not able to find rate for WLAN interface %s\n", ifname);
 	  return WEIGHT_WLAN_11MB;
 	}
       
-      OLSR_PRINTF(1, "Bitrate %d\n", ifr.ifr_ifru.ifru_ivalue)
+      OLSR_PRINTF(1, "Bitrate %d\n", ifr.ifr_ifru.ifru_ivalue);
 
       //WEIGHT_WLAN_LOW,          /* <11Mb WLAN     */
       //WEIGHT_WLAN_11MB,         /* 11Mb 802.11b   */
@@ -897,26 +801,26 @@ calculate_if_metric(char *ifname)
       memset(&ifr, 0, sizeof(ifr));
       strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
 
-      if (ioctl(ioctl_s, SIOCGMIIPHY, &ifr) < 0) {
+      if (ioctl(olsr_cnf->ioctl_s, SIOCGMIIPHY, &ifr) < 0) {
 	if (errno != ENODEV)
 	  OLSR_PRINTF(1, "SIOCGMIIPHY on '%s' failed: %s\n",
-		      ifr.ifr_name, strerror(errno))
+		      ifr.ifr_name, strerror(errno));
 	return WEIGHT_ETHERNET_DEFAULT;
       }
 
       mii->reg_num = MII_BMCR;
-      if (ioctl(ioctl_s, SIOCGMIIREG, &ifr) < 0) {
+      if (ioctl(olsr_cnf->ioctl_s, SIOCGMIIREG, &ifr) < 0) {
 	OLSR_PRINTF(1, "SIOCGMIIREG on %s failed: %s\n", ifr.ifr_name,
-		    strerror(errno))
+		    strerror(errno));
 	return WEIGHT_ETHERNET_DEFAULT;
       }
       bmcr = mii->val_out;
 
 
-      OLSR_PRINTF(1, "%s: ", ifr.ifr_name)
+      OLSR_PRINTF(1, "%s: ", ifr.ifr_name);
       OLSR_PRINTF(1, "%s Mbit, %s duplex\n",
 		  (bmcr & MII_BMCR_100MBIT) ? "100" : "10",
-		  (bmcr & MII_BMCR_DUPLEX) ? "full" : "half")
+		  (bmcr & MII_BMCR_DUPLEX) ? "full" : "half");
     
       is_if_link_up(ifname);
 
@@ -947,22 +851,22 @@ is_if_link_up(char *ifname)
       memset(&ifr, 0, sizeof(ifr));
       strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
 
-      if (ioctl(ioctl_s, SIOCGMIIPHY, &ifr) < 0) {
+      if (ioctl(olsr_cnf->ioctl_s, SIOCGMIIPHY, &ifr) < 0) {
 	if (errno != ENODEV)
 	  OLSR_PRINTF(1, "SIOCGMIIPHY on '%s' failed: %s\n",
-		      ifr.ifr_name, strerror(errno))
+		      ifr.ifr_name, strerror(errno));
 	return WEIGHT_ETHERNET_DEFAULT;
       }
       mii->reg_num = MII_BMSR;
-      if (ioctl(ioctl_s, SIOCGMIIREG, &ifr) < 0) {
+      if (ioctl(olsr_cnf->ioctl_s, SIOCGMIIREG, &ifr) < 0) {
 	OLSR_PRINTF(1, "SIOCGMIIREG on %s failed: %s\n", ifr.ifr_name,
-		    strerror(errno))
+		    strerror(errno));
 	return WEIGHT_ETHERNET_DEFAULT;
       }
       bmsr = mii->val_out;
 
-      OLSR_PRINTF(1, "%s: ", ifr.ifr_name)
-      OLSR_PRINTF(1, "%s\n", (bmsr & MII_BMSR_LINK_VALID) ? "link ok " : "no link ")
+      OLSR_PRINTF(1, "%s: ", ifr.ifr_name);
+      OLSR_PRINTF(1, "%s\n", (bmsr & MII_BMSR_LINK_VALID) ? "link ok " : "no link ");
     
       return (bmsr & MII_BMSR_LINK_VALID);
 
