@@ -36,7 +36,7 @@
  * to the project. For more information see the website or contact
  * the copyright holders.
  *
- * $Id: olsr.c,v 1.47 2005/11/17 04:25:44 tlopatic Exp $
+ * $Id: olsr.c,v 1.55 2007/07/05 22:43:47 bernd67 Exp $
  */
 
 /**
@@ -60,6 +60,7 @@
 #include "neighbor_table.h"
 #include "log.h"
 #include "lq_packet.h"
+#include "lq_avl.h"
 
 #include <stdarg.h>
 #include <signal.h>
@@ -68,6 +69,7 @@
 olsr_bool changes_topology;
 olsr_bool changes_neighborhood;
 olsr_bool changes_hna;
+olsr_bool changes_force;
 
 /**
  * Process changes functions
@@ -87,7 +89,7 @@ static olsr_u16_t message_seqno;
  *Initialize the message sequence number as a random value
  */
 void
-init_msg_seqno()
+init_msg_seqno(void)
 {
   message_seqno = random() & 0xFFFF;
 }
@@ -97,8 +99,8 @@ init_msg_seqno()
  *
  *@return the seqno
  */
-inline olsr_u16_t
-get_msg_seqno()
+olsr_u16_t
+get_msg_seqno(void)
 {
   return message_seqno++;
 }
@@ -109,7 +111,7 @@ register_pcf(int (*f)(int, int, int))
 {
   struct pcf *new_pcf;
 
-  OLSR_PRINTF(1, "Registering pcf function\n")
+  OLSR_PRINTF(1, "Registering pcf function\n");
 
   new_pcf = olsr_malloc(sizeof(struct pcf), "New PCF");
 
@@ -128,20 +130,24 @@ register_pcf(int (*f)(int, int, int))
  *@return 0
  */
 void
-olsr_process_changes()
+olsr_process_changes(void)
 {
-
   struct pcf *tmp_pc_list;
 
 #ifdef DEBUG
   if(changes_neighborhood)
-    OLSR_PRINTF(3, "CHANGES IN NEIGHBORHOOD\n")
+    OLSR_PRINTF(3, "CHANGES IN NEIGHBORHOOD\n");
   if(changes_topology)
-    OLSR_PRINTF(3, "CHANGES IN TOPOLOGY\n")
+    OLSR_PRINTF(3, "CHANGES IN TOPOLOGY\n");
   if(changes_hna)
-    OLSR_PRINTF(3, "CHANGES IN HNA\n")
+    OLSR_PRINTF(3, "CHANGES IN HNA\n");
 #endif
   
+  if(!changes_force &&
+     2 <= olsr_cnf->lq_level &&
+     0 >= olsr_cnf->lq_dlimit)
+    return;
+    
   if(!changes_neighborhood &&
      !changes_topology &&
      !changes_hna)
@@ -171,11 +177,6 @@ olsr_process_changes()
           olsr_calculate_routing_table();
           olsr_calculate_hna_routes();
         }
-
-      else
-        {
-          olsr_calculate_lq_routing_table();
-        }
     }
   
   else if (changes_topology)
@@ -187,11 +188,6 @@ olsr_process_changes()
           olsr_calculate_routing_table();
           olsr_calculate_hna_routes();
         }
-
-      else
-        {
-          olsr_calculate_lq_routing_table();
-        }
     }
 
   else if (changes_hna)
@@ -202,11 +198,11 @@ olsr_process_changes()
         {
           olsr_calculate_hna_routes();
         }
-
-      else
-        {
-          olsr_calculate_lq_routing_table();
-        }
+    }
+  
+  if (olsr_cnf->lq_level >= 2)
+    {
+      olsr_calculate_lq_routing_table();
     }
   
   if (olsr_cnf->debug_level > 0)
@@ -217,13 +213,17 @@ olsr_process_changes()
 	  
           if (olsr_cnf->debug_level > 3)
             {
-              olsr_print_duplicate_table();
+             if (olsr_cnf->debug_level > 8)
+               {
+                 olsr_print_duplicate_table();
+               }
               olsr_print_hna_set();
             }
         }
       
       olsr_print_link_set();
       olsr_print_neighbor_table();
+      olsr_print_two_hop_neighbor_table();
       olsr_print_tc_table();
     }
 
@@ -239,6 +239,7 @@ olsr_process_changes()
   changes_neighborhood = OLSR_FALSE;
   changes_topology = OLSR_FALSE;
   changes_hna = OLSR_FALSE;
+  changes_force = OLSR_FALSE;
 
 
   return;
@@ -254,13 +255,19 @@ olsr_process_changes()
  *Also initalizes other variables
  */
 void
-olsr_init_tables()
-{
-  
+olsr_init_tables(void)
+{  
   changes_topology = OLSR_FALSE;
   changes_neighborhood = OLSR_FALSE;
   changes_hna = OLSR_FALSE;
 
+  /* Set avl tree comparator */
+  if (olsr_cnf->ipsize == 4) {
+    avl_comp_default = 0;
+  } else {
+    avl_comp_default = avl_comp_ipv6;
+  }
+
   /* Initialize link set */
   olsr_init_link_set();
 
@@ -289,15 +296,9 @@ olsr_init_tables()
   olsr_init_mid_set();
 
   /* Initialize HNA set */
-  olsr_init_hna_set();
-  
+  olsr_init_hna_set();  
 }
 
-
-
-
-
-
 /**
  *Check if a message is to be forwarded and forward
  *it if necessary.
@@ -324,17 +325,18 @@ olsr_forward_message(union olsr_message *m,
   if(!olsr_check_dup_table_fwd(originator, seqno, &in_if->ip_addr))
     {
 #ifdef DEBUG
-      OLSR_PRINTF(3, "Message already forwarded!\n")
+      OLSR_PRINTF(3, "Message already forwarded!\n");
 #endif
       return 0;
     }
 
   /* Lookup sender address */
-  if(!(src = mid_lookup_main_addr(from_addr)))
+  src = mid_lookup_main_addr(from_addr);
+  if(!src)
     src = from_addr;
 
-
-  if(NULL == (neighbor=olsr_lookup_neighbor_table(src)))
+  neighbor=olsr_lookup_neighbor_table(src);
+  if(!neighbor)
     return 0;
 
   if(neighbor->status != SYM)
@@ -348,12 +350,11 @@ olsr_forward_message(union olsr_message *m,
   if(olsr_lookup_mprs_set(src) == NULL)
     {
 #ifdef DEBUG
-      OLSR_PRINTF(5, "Forward - sender %s not MPR selector\n", olsr_ip_to_string(src))
+      OLSR_PRINTF(5, "Forward - sender %s not MPR selector\n", olsr_ip_to_string(src));
 #endif
       return 0;
     }
 
-
   /* Treat TTL hopcnt */
   if(olsr_cnf->ip_version == AF_INET)
     {
@@ -368,14 +369,10 @@ olsr_forward_message(union olsr_message *m,
       m->v6.ttl--; 
     }
 
-
-
   /* Update dup forwarded */
   olsr_set_dup_forward(originator, seqno);
 
   /* Update packet data */
-
-
   msgsize = ntohs(m->v4.olsr_msgsize);
 
   /* looping trough interfaces */
@@ -386,37 +383,33 @@ olsr_forward_message(union olsr_message *m,
 	  /*
 	   * Check if message is to big to be piggybacked
 	   */
-	  if(net_outbuffer_push(ifn, (olsr_u8_t *)m, msgsize) != msgsize)
+	  if(net_outbuffer_push(ifn, m, msgsize) != msgsize)
 	    {
 	      /* Send */
 	      net_output(ifn);
 	      /* Buffer message */
 	      set_buffer_timer(ifn);
 	      
-	      if(net_outbuffer_push(ifn, (olsr_u8_t *)m, msgsize) != msgsize)
+	      if(net_outbuffer_push(ifn, m, msgsize) != msgsize)
 		{
-		  OLSR_PRINTF(1, "Received message to big to be forwarded in %s(%d bytes)!", ifn->int_name, msgsize)
+		  OLSR_PRINTF(1, "Received message to big to be forwarded in %s(%d bytes)!", ifn->int_name, msgsize);
 		  olsr_syslog(OLSR_LOG_ERR, "Received message to big to be forwarded on %s(%d bytes)!", ifn->int_name, msgsize);
 		}
-
 	    }
 	}
-      
       else
 	{
 	  /* No forwarding pending */
 	  set_buffer_timer(ifn);
 	  
-	  if(net_outbuffer_push(ifn, (olsr_u8_t *)m, msgsize) != msgsize)
+	  if(net_outbuffer_push(ifn, m, msgsize) != msgsize)
 	    {
-	      OLSR_PRINTF(1, "Received message to big to be forwarded in %s(%d bytes)!", ifn->int_name, msgsize)
+	      OLSR_PRINTF(1, "Received message to big to be forwarded in %s(%d bytes)!", ifn->int_name, msgsize);
 	      olsr_syslog(OLSR_LOG_ERR, "Received message to big to be forwarded on %s(%d bytes)!", ifn->int_name, msgsize);
 	    }
 	}
     }
-
   return 1;
-
 }
 
 
@@ -427,35 +420,30 @@ set_buffer_timer(struct interface *ifn)
       
   /* Set timer */
   jitter = (float) random()/RAND_MAX;
-  jitter *= max_jitter;
-
-  fwdtimer[ifn->if_nr] = GET_TIMESTAMP(jitter*1000);
+  jitter *= olsr_cnf->max_jitter;
 
+  ifn->fwdtimer = GET_TIMESTAMP(jitter*1000);
 }
 
-
-
 void
-olsr_init_willingness()
+olsr_init_willingness(void)
 {
   if(olsr_cnf->willingness_auto)
     olsr_register_scheduler_event(&olsr_update_willingness, 
-				  NULL, will_int, will_int, NULL);
+				  NULL, olsr_cnf->will_int, olsr_cnf->will_int, NULL);
 }
 
 void
-olsr_update_willingness(void *foo)
+olsr_update_willingness(void *foo __attribute__((unused)))
 {
-  int tmp_will;
-
-  tmp_will = olsr_cnf->willingness;
+  int tmp_will = olsr_cnf->willingness;
 
   /* Re-calculate willingness */
   olsr_cnf->willingness = olsr_calculate_willingness();
 
   if(tmp_will != olsr_cnf->willingness)
     {
-      OLSR_PRINTF(1, "Local willingness updated: old %d new %d\n", tmp_will, olsr_cnf->willingness)
+      OLSR_PRINTF(1, "Local willingness updated: old %d new %d\n", tmp_will, olsr_cnf->willingness);
     }
 }
 
@@ -469,7 +457,7 @@ olsr_update_willingness(void *foo)
  */
 
 olsr_u8_t
-olsr_calculate_willingness()
+olsr_calculate_willingness(void)
 {
   struct olsr_apm_info ainfo;
 
@@ -581,10 +569,10 @@ olsr_status_to_string(olsr_u8_t status)
 void
 olsr_exit(const char *msg, int val)
 {
-  OLSR_PRINTF(1, "OLSR EXIT: %s\n", msg)
+  OLSR_PRINTF(1, "OLSR EXIT: %s\n", msg);
   olsr_syslog(OLSR_LOG_ERR, "olsrd exit: %s\n", msg);
   fflush(stdout);
-  exit_value = val;
+  olsr_cnf->exit_value = val;
 
   raise(SIGTERM);
 }
@@ -606,9 +594,10 @@ olsr_malloc(size_t size, const char *id)
 
   if((ptr = malloc(size)) == 0) 
     {
-      OLSR_PRINTF(1, "OUT OF MEMORY: %s\n", strerror(errno))
-      olsr_syslog(OLSR_LOG_ERR, "olsrd: out of memory!: %m\n");
-      olsr_exit((char *)id, EXIT_FAILURE);
+      const char * const err_msg = strerror(errno);
+      OLSR_PRINTF(1, "OUT OF MEMORY: %s\n", err_msg);
+      olsr_syslog(OLSR_LOG_ERR, "olsrd: out of memory!: %s\n", err_msg);
+      olsr_exit(id, EXIT_FAILURE);
     }
   return ptr;
 }
@@ -623,17 +612,12 @@ olsr_malloc(size_t size, const char *id)
 int
 olsr_printf(int loglevel, char *format, ...)
 {
-  va_list arglist;
-
   if((loglevel <= olsr_cnf->debug_level) && debug_handle)
     {
+      va_list arglist;
       va_start(arglist, format);
-      
       vfprintf(debug_handle, format, arglist);
-      
       va_end(arglist);
     }
-
-
   return 0;
 }
