@@ -36,7 +36,7 @@
  * to the project. For more information see the website or contact
  * the copyright holders.
  *
- * $Id: ifnet.c,v 1.31 2005/12/29 18:37:16 tlopatic Exp $
+ * $Id: ifnet.c,v 1.49 2007/05/13 22:23:55 bernd67 Exp $
  */
 
 
@@ -49,6 +49,7 @@
 #include "defs.h"
 #include "olsr.h"
 #include "net_os.h"
+#include "net_olsr.h"
 #include "socket_parser.h"
 #include "parser.h"
 #include "scheduler.h"
@@ -56,6 +57,7 @@
 #include "mantissa.h"
 #include "lq_packet.h"
 #include "log.h"
+#include "link_set.h"
 #include <signal.h>
 #include <sys/types.h>
 #include <net/if.h>
@@ -70,14 +72,14 @@ int bufspace = 127*1024;	/* max. input buffer size to request */
 
 
 int
-set_flag(char *ifname, short flag)
+set_flag(char *ifname, short flag __attribute__((unused)))
 {
   struct ifreq ifr;
 
   strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
 
   /* Get flags */
-  if (ioctl(ioctl_s, SIOCGIFFLAGS, &ifr) < 0) 
+  if (ioctl(olsr_cnf->ioctl_s, SIOCGIFFLAGS, &ifr) < 0) 
     {
       fprintf(stderr,"ioctl (get interface flags)");
       return -1;
@@ -92,7 +94,7 @@ set_flag(char *ifname, short flag)
       /* Add UP */
       ifr.ifr_flags |= (IFF_UP | IFF_RUNNING);
       /* Set flags + UP */
-      if(ioctl(ioctl_s, SIOCSIFFLAGS, &ifr) < 0)
+      if(ioctl(olsr_cnf->ioctl_s, SIOCSIFFLAGS, &ifr) < 0)
 	{
 	  fprintf(stderr, "ERROR(%s): %s\n", ifr.ifr_name, strerror(errno));
 	  return -1;
@@ -106,12 +108,12 @@ set_flag(char *ifname, short flag)
 
 
 void
-check_interface_updates(void *foo)
+check_interface_updates(void *foo __attribute__((unused)))
 {
   struct olsr_if *tmp_if;
 
 #ifdef DEBUG
-  OLSR_PRINTF(3, "Checking for updates in the interface set\n")
+  OLSR_PRINTF(3, "Checking for updates in the interface set\n");
 #endif
 
   for(tmp_if = olsr_cnf->interfaces; tmp_if != NULL; tmp_if = tmp_if->next)
@@ -122,10 +124,23 @@ check_interface_updates(void *foo)
       if(olsr_cnf->host_emul) /* XXX: TEMPORARY! */
 	continue;
 
+      if(!tmp_if->cnf->autodetect_chg) 
+        {
+#ifdef DEBUG
+          /* Don't check this interface */
+          OLSR_PRINTF(3, "Not checking interface %s\n", tmp_if->name)
+#endif
+          continue;
+        }
+
       if(tmp_if->configured)
-	chk_if_changed(tmp_if);
+        {
+          chk_if_changed(tmp_if);
+        }
       else
-	chk_if_up(tmp_if, 3);
+        {
+          chk_if_up(tmp_if, 3);
+        }
     }
 
   return;
@@ -148,7 +163,7 @@ chk_if_changed(struct olsr_if *iface)
   if_changes = 0;
 
 #ifdef DEBUG
-  OLSR_PRINTF(3, "Checking if %s is set down or changed\n", iface->name)
+  OLSR_PRINTF(3, "Checking if %s is set down or changed\n", iface->name);
 #endif
 
   if(iface->host_emul)
@@ -168,9 +183,9 @@ chk_if_changed(struct olsr_if *iface)
 
 
   /* Get flags (and check if interface exists) */
-  if (ioctl(ioctl_s, SIOCGIFFLAGS, &ifr) < 0) 
+  if (ioctl(olsr_cnf->ioctl_s, SIOCGIFFLAGS, &ifr) < 0) 
     {
-      OLSR_PRINTF(3, "No such interface: %s\n", iface->name)
+      OLSR_PRINTF(3, "No such interface: %s\n", iface->name);
       goto remove_interface;
     }
 
@@ -182,7 +197,7 @@ chk_if_changed(struct olsr_if *iface)
 
   if ((ifp->int_flags & IFF_UP) == 0)
     {
-      OLSR_PRINTF(1, "\tInterface %s not up - removing it...\n", iface->name)
+      OLSR_PRINTF(1, "\tInterface %s not up - removing it...\n", iface->name);
       goto remove_interface;
     }
 
@@ -197,13 +212,13 @@ chk_if_changed(struct olsr_if *iface)
       !iface->cnf->ipv4_broadcast.v4 && /* Skip if fixed bcast */ 
       (!(ifp->int_flags & IFF_BROADCAST))) 
     {
-      OLSR_PRINTF(3, "\tNo broadcast - removing\n")
+      OLSR_PRINTF(3, "\tNo broadcast - removing\n");
       goto remove_interface;
     }
 
   if (ifp->int_flags & IFF_LOOPBACK)
     {
-      OLSR_PRINTF(3, "\tThis is a loopback interface - removing it...\n")
+      OLSR_PRINTF(3, "\tThis is a loopback interface - removing it...\n");
       goto remove_interface;
     }
 
@@ -219,7 +234,7 @@ chk_if_changed(struct olsr_if *iface)
     ifp->int_metric = calculate_if_metric(ifr.ifr_name);
 
   /* Get MTU */
-  if (ioctl(ioctl_s, SIOCGIFMTU, &ifr) < 0)
+  if (ioctl(olsr_cnf->ioctl_s, SIOCGIFMTU, &ifr) < 0)
     ifp->int_mtu = 0;
   else
     {
@@ -250,34 +265,34 @@ chk_if_changed(struct olsr_if *iface)
       if(get_ipv6_address(ifr.ifr_name, &tmp_saddr6, iface->cnf->ipv6_addrtype) <= 0)
 	{
 	  if(iface->cnf->ipv6_addrtype == IPV6_ADDR_SITELOCAL)
-	    OLSR_PRINTF(3, "\tCould not find site-local IPv6 address for %s\n", ifr.ifr_name)
+	    OLSR_PRINTF(3, "\tCould not find site-local IPv6 address for %s\n", ifr.ifr_name);
 	  else
-	    OLSR_PRINTF(3, "\tCould not find global IPv6 address for %s\n", ifr.ifr_name)
+	    OLSR_PRINTF(3, "\tCould not find global IPv6 address for %s\n", ifr.ifr_name);
 	  
 	  
 	  goto remove_interface;
 	}
       
 #ifdef DEBUG
-      OLSR_PRINTF(3, "\tAddress: %s\n", ip6_to_string(&tmp_saddr6.sin6_addr))
+      OLSR_PRINTF(3, "\tAddress: %s\n", ip6_to_string(&tmp_saddr6.sin6_addr));
 #endif
 
-      if(memcmp(&tmp_saddr6.sin6_addr, &ifp->int6_addr.sin6_addr, ipsize) != 0)
+      if(memcmp(&tmp_saddr6.sin6_addr, &ifp->int6_addr.sin6_addr, olsr_cnf->ipsize) != 0)
 	{
-	  OLSR_PRINTF(1, "New IP address for %s:\n", ifr.ifr_name)
-	  OLSR_PRINTF(1, "\tOld: %s\n", ip6_to_string(&ifp->int6_addr.sin6_addr))
-	  OLSR_PRINTF(1, "\tNew: %s\n", ip6_to_string(&tmp_saddr6.sin6_addr))
+	  OLSR_PRINTF(1, "New IP address for %s:\n", ifr.ifr_name);
+	  OLSR_PRINTF(1, "\tOld: %s\n", ip6_to_string(&ifp->int6_addr.sin6_addr));
+	  OLSR_PRINTF(1, "\tNew: %s\n", ip6_to_string(&tmp_saddr6.sin6_addr));
 
 	  /* Check main addr */
-	  if(memcmp(&main_addr, &tmp_saddr6.sin6_addr, ipsize) == 0)
+	  if(memcmp(&olsr_cnf->main_addr, &tmp_saddr6.sin6_addr, olsr_cnf->ipsize) == 0)
 	    {
 	      /* Update main addr */
-	      memcpy(&main_addr, &tmp_saddr6.sin6_addr, ipsize);
+	      memcpy(&olsr_cnf->main_addr, &tmp_saddr6.sin6_addr, olsr_cnf->ipsize);
 	    }
 
 	  /* Update address */
-	  memcpy(&ifp->int6_addr.sin6_addr, &tmp_saddr6.sin6_addr, ipsize);
-	  memcpy(&ifp->ip_addr, &tmp_saddr6.sin6_addr, ipsize);
+	  memcpy(&ifp->int6_addr.sin6_addr, &tmp_saddr6.sin6_addr, olsr_cnf->ipsize);
+	  memcpy(&ifp->ip_addr, &tmp_saddr6.sin6_addr, olsr_cnf->ipsize);
 
 	  run_ifchg_cbs(ifp, IFCHG_IF_UPDATE);
 
@@ -290,63 +305,64 @@ chk_if_changed(struct olsr_if *iface)
   /* IP version 4 */
     {
       /* Check interface address (IPv4)*/
-      if(ioctl(ioctl_s, SIOCGIFADDR, &ifr) < 0) 
+      if(ioctl(olsr_cnf->ioctl_s, SIOCGIFADDR, &ifr) < 0) 
 	{
-	  OLSR_PRINTF(1, "\tCould not get address of interface - removing it\n")
+	  OLSR_PRINTF(1, "\tCould not get address of interface - removing it\n");
 	  goto remove_interface;
 	}
 
 #ifdef DEBUG
-      OLSR_PRINTF(3, "\tAddress:%s\n", sockaddr_to_string(&ifr.ifr_addr))
+      OLSR_PRINTF(3, "\tAddress:%s\n", sockaddr_to_string(&ifr.ifr_addr));
 #endif
 
       if(memcmp(&((struct sockaddr_in *)&ifp->int_addr)->sin_addr.s_addr,
 		&((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr, 
-		ipsize) != 0)
+		olsr_cnf->ipsize) != 0)
 	{
 	  /* New address */
-	  OLSR_PRINTF(1, "IPv4 address changed for %s\n", ifr.ifr_name)
-	  OLSR_PRINTF(1, "\tOld:%s\n", sockaddr_to_string(&ifp->int_addr))
-	  OLSR_PRINTF(1, "\tNew:%s\n", sockaddr_to_string(&ifr.ifr_addr))
-	  
-	  if(memcmp(&main_addr, 
-		    &((struct sockaddr_in *)&ifp->int_addr)->sin_addr.s_addr, 
-		    ipsize) == 0)
-	    {
-	      OLSR_PRINTF(1, "New main address: %s\n", sockaddr_to_string(&ifr.ifr_addr))
-	      olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", sockaddr_to_string(&ifr.ifr_addr));
-	      memcpy(&main_addr, 
-		     &((struct sockaddr_in *)&ifp->int_addr)->sin_addr.s_addr, 
-		     ipsize);
-	    }
+	  OLSR_PRINTF(1, "IPv4 address changed for %s\n", ifr.ifr_name);
+	  OLSR_PRINTF(1, "\tOld:%s\n", sockaddr_to_string(&ifp->int_addr));
+	  OLSR_PRINTF(1, "\tNew:%s\n", sockaddr_to_string(&ifr.ifr_addr));
 
 	  ifp->int_addr = ifr.ifr_addr;
+
+	  if(memcmp(&olsr_cnf->main_addr, 
+		    &ifp->ip_addr,
+		    olsr_cnf->ipsize) == 0)
+	    {
+	      OLSR_PRINTF(1, "New main address: %s\n", sockaddr_to_string(&ifr.ifr_addr));
+	      olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", sockaddr_to_string(&ifr.ifr_addr));
+	      memcpy(&olsr_cnf->main_addr, 
+		     &((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr, 
+		     olsr_cnf->ipsize);
+	    }
+
 	  memcpy(&ifp->ip_addr, 
-		 &((struct sockaddr_in *)&ifp->int_addr)->sin_addr.s_addr, 
-		 ipsize);
+		 &((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr, 
+		 olsr_cnf->ipsize);
 
 	  if_changes = 1;
 	}
 
       /* Check netmask */
-      if (ioctl(ioctl_s, SIOCGIFNETMASK, &ifr) < 0) 
+      if (ioctl(olsr_cnf->ioctl_s, SIOCGIFNETMASK, &ifr) < 0) 
 	{
 	  olsr_syslog(OLSR_LOG_ERR, "%s: ioctl (get broadaddr)", ifr.ifr_name);
 	  goto remove_interface;
 	}
 
 #ifdef DEBUG
-      OLSR_PRINTF(3, "\tNetmask:%s\n", sockaddr_to_string(&ifr.ifr_netmask))
+      OLSR_PRINTF(3, "\tNetmask:%s\n", sockaddr_to_string(&ifr.ifr_netmask));
 #endif
 
       if(memcmp(&((struct sockaddr_in *)&ifp->int_netmask)->sin_addr.s_addr,
 		&((struct sockaddr_in *)&ifr.ifr_netmask)->sin_addr.s_addr, 
-		ipsize) != 0)
+		olsr_cnf->ipsize) != 0)
 	{
 	  /* New address */
-	  OLSR_PRINTF(1, "IPv4 netmask changed for %s\n", ifr.ifr_name)
-	  OLSR_PRINTF(1, "\tOld:%s\n", sockaddr_to_string(&ifp->int_netmask))
-	  OLSR_PRINTF(1, "\tNew:%s\n", sockaddr_to_string(&ifr.ifr_netmask))
+	  OLSR_PRINTF(1, "IPv4 netmask changed for %s\n", ifr.ifr_name);
+	  OLSR_PRINTF(1, "\tOld:%s\n", sockaddr_to_string(&ifp->int_netmask));
+	  OLSR_PRINTF(1, "\tNew:%s\n", sockaddr_to_string(&ifr.ifr_netmask));
 
 	  ifp->int_netmask = ifr.ifr_netmask;
 
@@ -356,25 +372,25 @@ chk_if_changed(struct olsr_if *iface)
       if(!iface->cnf->ipv4_broadcast.v4)
 	{
 	  /* Check broadcast address */      
-	  if (ioctl(ioctl_s, SIOCGIFBRDADDR, &ifr) < 0) 
+	  if (ioctl(olsr_cnf->ioctl_s, SIOCGIFBRDADDR, &ifr) < 0) 
 	    {
 	      olsr_syslog(OLSR_LOG_ERR, "%s: ioctl (get broadaddr)", ifr.ifr_name);
 	      goto remove_interface;
 	    }
 	  
 #ifdef DEBUG
-	  OLSR_PRINTF(3, "\tBroadcast address:%s\n", sockaddr_to_string(&ifr.ifr_broadaddr))
+	  OLSR_PRINTF(3, "\tBroadcast address:%s\n", sockaddr_to_string(&ifr.ifr_broadaddr));
 #endif
 	  
 	  if(memcmp(&((struct sockaddr_in *)&ifp->int_broadaddr)->sin_addr.s_addr,
 		    &((struct sockaddr_in *)&ifr.ifr_broadaddr)->sin_addr.s_addr, 
-		    ipsize) != 0)
+		    olsr_cnf->ipsize) != 0)
 	    {
 	      
 	      /* New address */
-	      OLSR_PRINTF(1, "IPv4 broadcast changed for %s\n", ifr.ifr_name)
-	      OLSR_PRINTF(1, "\tOld:%s\n", sockaddr_to_string(&ifp->int_broadaddr))
-	      OLSR_PRINTF(1, "\tNew:%s\n", sockaddr_to_string(&ifr.ifr_broadaddr))
+	      OLSR_PRINTF(1, "IPv4 broadcast changed for %s\n", ifr.ifr_name);
+	      OLSR_PRINTF(1, "\tOld:%s\n", sockaddr_to_string(&ifp->int_broadaddr));
+	      OLSR_PRINTF(1, "\tNew:%s\n", sockaddr_to_string(&ifr.ifr_broadaddr));
 	      
 	      ifp->int_broadaddr = ifr.ifr_broadaddr;
 	      if_changes = 1;
@@ -389,9 +405,11 @@ chk_if_changed(struct olsr_if *iface)
 
 
  remove_interface:
-  OLSR_PRINTF(1, "Removing interface %s\n", iface->name)
+  OLSR_PRINTF(1, "Removing interface %s\n", iface->name);
   olsr_syslog(OLSR_LOG_INFO, "Removing interface %s\n", iface->name);
 
+  del_if_link_entries(&ifp->ip_addr);
+
   /*
    *Call possible ifchange functions registered by plugins  
    */
@@ -417,19 +435,19 @@ chk_if_changed(struct olsr_if *iface)
   net_remove_buffer(ifp);
 
   /* Check main addr */
-  if(COMP_IP(&main_addr, &ifp->ip_addr))
+  if(COMP_IP(&olsr_cnf->main_addr, &ifp->ip_addr))
     {
       if(ifnet == NULL)
 	{
 	  /* No more interfaces */
-	  memset(&main_addr, 0, ipsize);
-	  OLSR_PRINTF(1, "No more interfaces...\n")
+	  memset(&olsr_cnf->main_addr, 0, olsr_cnf->ipsize);
+	  OLSR_PRINTF(1, "No more interfaces...\n");
 	}
       else
 	{
-	  COPY_IP(&main_addr, &ifnet->ip_addr);
-	  OLSR_PRINTF(1, "New main address: %s\n", olsr_ip_to_string(&main_addr))
-	  olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", olsr_ip_to_string(&main_addr));
+	  COPY_IP(&olsr_cnf->main_addr, &ifnet->ip_addr);
+	  OLSR_PRINTF(1, "New main address: %s\n", olsr_ip_to_string(&olsr_cnf->main_addr));
+	  olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", olsr_ip_to_string(&olsr_cnf->main_addr));
 	}
     }
 
@@ -484,15 +502,16 @@ chk_if_changed(struct olsr_if *iface)
   /* Close olsr socket */
   close(ifp->olsr_socket);
   remove_olsr_socket(ifp->olsr_socket, &olsr_input);
+
   /* Free memory */
   free(ifp->int_name);
   free(ifp);
 
   if((ifnet == NULL) && (!olsr_cnf->allow_no_interfaces))
     {
-      OLSR_PRINTF(1, "No more active interfaces - exiting.\n")
+      OLSR_PRINTF(1, "No more active interfaces - exiting.\n");
       olsr_syslog(OLSR_LOG_INFO, "No more active interfaces - exiting.\n");
-      exit_value = EXIT_FAILURE;
+      olsr_cnf->exit_value = EXIT_FAILURE;
       kill(getpid(), SIGINT);
     }
 
@@ -527,28 +546,23 @@ add_hemu_if(struct olsr_if *iface)
 
   strcpy(ifp->int_name, "hcif01");
 
-  OLSR_PRINTF(1, "Adding %s(host emulation):\n", ifp->int_name)
+  OLSR_PRINTF(1, "Adding %s(host emulation):\n", ifp->int_name);
 
   OLSR_PRINTF(1, "       Address:%s\n", olsr_ip_to_string(&iface->hemu_ip));
 
-  OLSR_PRINTF(1, "       Index:%d\n", iface->index);
-
   OLSR_PRINTF(1, "       NB! This is a emulated interface\n       that does not exist in the kernel!\n");
 
   ifp->int_next = ifnet;
   ifnet = ifp;
 
-  memset(&null_addr, 0, ipsize);
-  if(COMP_IP(&null_addr, &main_addr))
+  memset(&null_addr, 0, olsr_cnf->ipsize);
+  if(COMP_IP(&null_addr, &olsr_cnf->main_addr))
     {
-      COPY_IP(&main_addr, &iface->hemu_ip);
-      OLSR_PRINTF(1, "New main address: %s\n", olsr_ip_to_string(&main_addr))
-	olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", olsr_ip_to_string(&main_addr));
+      COPY_IP(&olsr_cnf->main_addr, &iface->hemu_ip);
+      OLSR_PRINTF(1, "New main address: %s\n", olsr_ip_to_string(&olsr_cnf->main_addr));
+	olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", olsr_ip_to_string(&olsr_cnf->main_addr));
     }
 
-  /* setting the interfaces number*/
-  ifp->if_nr = iface->index;
-
   ifp->int_mtu = OLSR_DEFAULT_MTU;
 
   ifp->int_mtu -= (olsr_cnf->ip_version == AF_INET6) ? UDP_IPV6_HDRSIZE : UDP_IPV4_HDRSIZE;
@@ -570,7 +584,7 @@ add_hemu_if(struct olsr_if *iface)
      /* IP version 4 */
       ifp->ip_addr.v4 = iface->hemu_ip.v4;
 
-      memcpy(&((struct sockaddr_in *)&ifp->int_addr)->sin_addr, &iface->hemu_ip, ipsize);
+      memcpy(&((struct sockaddr_in *)&ifp->int_addr)->sin_addr, &iface->hemu_ip, olsr_cnf->ipsize);
       
       /*
        *We create one socket for each interface and bind
@@ -584,7 +598,7 @@ add_hemu_if(struct olsr_if *iface)
 	{
 	  fprintf(stderr, "Could not initialize socket... exiting!\n\n");
 	  olsr_syslog(OLSR_LOG_ERR, "Could not initialize socket... exiting!\n\n");
-	  exit_value = EXIT_FAILURE;
+	  olsr_cnf->exit_value = EXIT_FAILURE;
 	  kill(getpid(), SIGINT);
 	}
 
@@ -592,7 +606,7 @@ add_hemu_if(struct olsr_if *iface)
   else
     {
       /* IP version 6 */
-      memcpy(&ifp->ip_addr, &iface->hemu_ip, ipsize);
+      memcpy(&ifp->ip_addr, &iface->hemu_ip, olsr_cnf->ipsize);
 
 #if 0      
       /*
@@ -609,7 +623,7 @@ add_hemu_if(struct olsr_if *iface)
 	{
 	  fprintf(stderr, "Could not initialize socket... exiting!\n\n");
 	  olsr_syslog(OLSR_LOG_ERR, "Could not initialize socket... exiting!\n\n");
-	  exit_value = EXIT_FAILURE;
+	  olsr_cnf->exit_value = EXIT_FAILURE;
 	  kill(getpid(), SIGINT);
 	}
       
@@ -617,13 +631,13 @@ add_hemu_if(struct olsr_if *iface)
     }
 
   /* Send IP as first 4/16 bytes on socket */
-  memcpy(addr, iface->hemu_ip.v6.s6_addr, ipsize);
+  memcpy(addr, iface->hemu_ip.v6.s6_addr, olsr_cnf->ipsize);
   addr[0] = htonl(addr[0]);
   addr[1] = htonl(addr[1]);
   addr[2] = htonl(addr[2]);
   addr[3] = htonl(addr[3]);
 
-  if(send(ifp->olsr_socket, addr , ipsize, 0) != (int)ipsize)
+  if(send(ifp->olsr_socket, addr , olsr_cnf->ipsize, 0) != (int)olsr_cnf->ipsize)
     {
       fprintf(stderr, "Error sending IP!");
     }  
@@ -673,12 +687,13 @@ add_hemu_if(struct olsr_if *iface)
 
   /* Recalculate max jitter */
 
-  if((max_jitter == 0) || ((iface->cnf->hello_params.emission_interval / 4) < max_jitter))
-    max_jitter = iface->cnf->hello_params.emission_interval / 4;
+  if((olsr_cnf->max_jitter == 0) || 
+     ((iface->cnf->hello_params.emission_interval / 4) < olsr_cnf->max_jitter))
+    olsr_cnf->max_jitter = iface->cnf->hello_params.emission_interval / 4;
 
   /* Recalculate max topology hold time */
-  if(max_tc_vtime < iface->cnf->tc_params.emission_interval)
-    max_tc_vtime = iface->cnf->tc_params.emission_interval;
+  if(olsr_cnf->max_tc_vtime < iface->cnf->tc_params.emission_interval)
+    olsr_cnf->max_tc_vtime = iface->cnf->tc_params.emission_interval;
 
   ifp->hello_etime = iface->cnf->hello_params.emission_interval;
   ifp->valtimes.hello = double_to_me(iface->cnf->hello_params.validity_time);
@@ -689,6 +704,17 @@ add_hemu_if(struct olsr_if *iface)
   return 1;
 }
 
+static char basename[32];
+char* if_basename(char* name);
+char* if_basename(char* name)
+{
+	char *p = strchr(name, ':');
+	if (0 == p || p - name >= (int)(sizeof(basename) / sizeof(basename[0]) - 1)) return name;
+	memcpy(basename, name, p - name);
+	basename[p - name] = 0;
+	return basename;
+}
+
 /**
  * Initializes a interface described by iface,
  * if it is set up and is of the correct type.
@@ -714,21 +740,22 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
   memset(&ifr, 0, sizeof(struct ifreq));
   strncpy(ifr.ifr_name, iface->name, IFNAMSIZ);
 
-  OLSR_PRINTF(debuglvl, "Checking %s:\n", ifr.ifr_name)
+  OLSR_PRINTF(debuglvl, "Checking %s:\n", ifr.ifr_name);
 
   /* Get flags (and check if interface exists) */
-  if (ioctl(ioctl_s, SIOCGIFFLAGS, &ifr) < 0) 
+  if (ioctl(olsr_cnf->ioctl_s, SIOCGIFFLAGS, &ifr) < 0) 
     {
-      OLSR_PRINTF(debuglvl, "\tNo such interface!\n")
+      OLSR_PRINTF(debuglvl, "\tNo such interface!\n");
       return 0;
     }
 
+  memset(&ifs.netbuf, 0, sizeof(struct olsr_netbuf));
   ifs.int_flags = ifr.ifr_flags;      
 
 
   if ((ifs.int_flags & IFF_UP) == 0)
     {
-      OLSR_PRINTF(debuglvl, "\tInterface not up - skipping it...\n")
+      OLSR_PRINTF(debuglvl, "\tInterface not up - skipping it...\n");
       return 0;
     }
 
@@ -737,14 +764,14 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
       !iface->cnf->ipv4_broadcast.v4 && /* Skip if fixed bcast */ 
       (!(ifs.int_flags & IFF_BROADCAST))) 
     {
-      OLSR_PRINTF(debuglvl, "\tNo broadcast - skipping\n")
+      OLSR_PRINTF(debuglvl, "\tNo broadcast - skipping\n");
       return 0;
     }
 
 
   if (ifs.int_flags & IFF_LOOPBACK)
     {
-      OLSR_PRINTF(debuglvl, "\tThis is a loopback interface - skipping it...\n")
+      OLSR_PRINTF(debuglvl, "\tThis is a loopback interface - skipping it...\n");
       return 0;
     }
 
@@ -754,9 +781,9 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
   ifs.is_wireless = check_wireless_interface(ifr.ifr_name);
 
   if(ifs.is_wireless)
-    OLSR_PRINTF(debuglvl, "\tWireless interface detected\n")
+    OLSR_PRINTF(debuglvl, "\tWireless interface detected\n");
   else
-    OLSR_PRINTF(debuglvl, "\tNot a wireless interface\n")
+    OLSR_PRINTF(debuglvl, "\tNot a wireless interface\n");
 
   
   /* IP version 6 */
@@ -767,14 +794,14 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
       if(get_ipv6_address(ifr.ifr_name, &ifs.int6_addr, iface->cnf->ipv6_addrtype) <= 0)
 	{
 	  if(iface->cnf->ipv6_addrtype == IPV6_ADDR_SITELOCAL)
-	    OLSR_PRINTF(debuglvl, "\tCould not find site-local IPv6 address for %s\n", ifr.ifr_name)
+	    OLSR_PRINTF(debuglvl, "\tCould not find site-local IPv6 address for %s\n", ifr.ifr_name);
 	  else
-	    OLSR_PRINTF(debuglvl, "\tCould not find global IPv6 address for %s\n", ifr.ifr_name)
+	    OLSR_PRINTF(debuglvl, "\tCould not find global IPv6 address for %s\n", ifr.ifr_name);
 	  
 	  return 0;
 	}
       
-      OLSR_PRINTF(debuglvl, "\tAddress: %s\n", ip6_to_string(&ifs.int6_addr.sin6_addr))
+      OLSR_PRINTF(debuglvl, "\tAddress: %s\n", ip6_to_string(&ifs.int6_addr.sin6_addr));
       
       /* Multicast */
       ifs.int6_multaddr.sin6_addr = (iface->cnf->ipv6_addrtype == IPV6_ADDR_SITELOCAL) ? 
@@ -783,18 +810,22 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
       /* Set address family */
       ifs.int6_multaddr.sin6_family = AF_INET6;
       /* Set port */
-      ifs.int6_multaddr.sin6_port = olsr_udp_port;
+      ifs.int6_multaddr.sin6_port = htons(OLSRPORT);
       
-      OLSR_PRINTF(debuglvl, "\tMulticast: %s\n", ip6_to_string(&ifs.int6_multaddr.sin6_addr))
+#ifdef __MacOSX__
+      ifs.int6_multaddr.sin6_scope_id = 0;
+#endif
+
+      OLSR_PRINTF(debuglvl, "\tMulticast: %s\n", ip6_to_string(&ifs.int6_multaddr.sin6_addr));
       
     }
   /* IP version 4 */
   else
     {
       /* Get interface address (IPv4)*/
-      if(ioctl(ioctl_s, SIOCGIFADDR, &ifr) < 0) 
+      if(ioctl(olsr_cnf->ioctl_s, SIOCGIFADDR, &ifr) < 0) 
 	{
-	  OLSR_PRINTF(debuglvl, "\tCould not get address of interface - skipping it\n")
+	  OLSR_PRINTF(debuglvl, "\tCould not get address of interface - skipping it\n");
 	  return 0;
 	}
       
@@ -802,7 +833,7 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
       
       /* Find netmask */
       
-      if (ioctl(ioctl_s, SIOCGIFNETMASK, &ifr) < 0) 
+      if (ioctl(olsr_cnf->ioctl_s, SIOCGIFNETMASK, &ifr) < 0) 
 	{
 	  olsr_syslog(OLSR_LOG_ERR, "%s: ioctl (get netmask)", ifr.ifr_name);
 	  return 0;
@@ -822,7 +853,7 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
       else
 	{
 	  /* Autodetect */
-	  if (ioctl(ioctl_s, SIOCGIFBRDADDR, &ifr) < 0) 
+	  if (ioctl(olsr_cnf->ioctl_s, SIOCGIFBRDADDR, &ifr) < 0) 
 	    {
 	      olsr_syslog(OLSR_LOG_ERR, "%s: ioctl (get broadaddr)", ifr.ifr_name);
 	      return 0;
@@ -832,10 +863,10 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
 	}
       
       /* Deactivate IP spoof filter */
-      deactivate_spoof(ifr.ifr_name, iface->index, olsr_cnf->ip_version);
+      deactivate_spoof(if_basename(ifr.ifr_name), &ifs, olsr_cnf->ip_version);
       
       /* Disable ICMP redirects */
-      disable_redirects(ifr.ifr_name, iface->index, olsr_cnf->ip_version);
+      disable_redirects(if_basename(ifr.ifr_name), &ifs, olsr_cnf->ip_version);
       
     }
   
@@ -844,48 +875,45 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
   
   ifs.if_index = if_nametoindex(ifr.ifr_name);
   
-  /* Set interface metric */
   /* Set interface metric */
   if(iface->cnf->weight.fixed)
     ifs.int_metric = iface->cnf->weight.value;
   else
     ifs.int_metric = calculate_if_metric(ifr.ifr_name);
-  OLSR_PRINTF(1, "\tMetric: %d\n", ifs.int_metric)
-
-  /* setting the interfaces number*/
-  ifs.if_nr = iface->index;
-
+  OLSR_PRINTF(1, "\tMetric: %d\n", ifs.int_metric);
 
   /* Get MTU */
-  if (ioctl(ioctl_s, SIOCGIFMTU, &ifr) < 0)
+  if (ioctl(olsr_cnf->ioctl_s, SIOCGIFMTU, &ifr) < 0)
     ifs.int_mtu = OLSR_DEFAULT_MTU;
   else
     ifs.int_mtu = ifr.ifr_mtu;
 
   ifs.int_mtu -= (olsr_cnf->ip_version == AF_INET6) ? UDP_IPV6_HDRSIZE : UDP_IPV4_HDRSIZE;
 
+  ifs.ttl_index = 0;
+
   /* Set up buffer */
   net_add_buffer(&ifs);
 	       
-  OLSR_PRINTF(1, "\tMTU - IPhdr: %d\n", ifs.int_mtu)
+  OLSR_PRINTF(1, "\tMTU - IPhdr: %d\n", ifs.int_mtu);
 
   olsr_syslog(OLSR_LOG_INFO, "Adding interface %s\n", iface->name);
-  OLSR_PRINTF(1, "\tIndex %d\n", ifs.if_nr)
+  OLSR_PRINTF(1, "\tIndex %d\n", ifs.if_index);
 
   if(olsr_cnf->ip_version == AF_INET)
     {
-      OLSR_PRINTF(1, "\tAddress:%s\n", sockaddr_to_string(&ifs.int_addr))
-      OLSR_PRINTF(1, "\tNetmask:%s\n", sockaddr_to_string(&ifs.int_netmask))
-      OLSR_PRINTF(1, "\tBroadcast address:%s\n", sockaddr_to_string(&ifs.int_broadaddr))
+      OLSR_PRINTF(1, "\tAddress:%s\n", sockaddr_to_string(&ifs.int_addr));
+      OLSR_PRINTF(1, "\tNetmask:%s\n", sockaddr_to_string(&ifs.int_netmask));
+      OLSR_PRINTF(1, "\tBroadcast address:%s\n", sockaddr_to_string(&ifs.int_broadaddr));
     }
   else
     {
-      OLSR_PRINTF(1, "\tAddress: %s\n", ip6_to_string(&ifs.int6_addr.sin6_addr))
-      OLSR_PRINTF(1, "\tMulticast: %s\n", ip6_to_string(&ifs.int6_multaddr.sin6_addr))
+      OLSR_PRINTF(1, "\tAddress: %s\n", ip6_to_string(&ifs.int6_addr.sin6_addr));
+      OLSR_PRINTF(1, "\tMulticast: %s\n", ip6_to_string(&ifs.int6_multaddr.sin6_addr));
     }
   
   ifp = olsr_malloc(sizeof (struct interface), "Interface update 2");
-  
+
   iface->configured = 1;
   iface->interf = ifp;
   
@@ -894,7 +922,7 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
   ifp->gen_properties = NULL;
   ifp->int_name = olsr_malloc(strlen(ifr.ifr_name) + 1, "Interface update 3");
       
-  strcpy(ifp->int_name, ifr.ifr_name);
+  strcpy(ifp->int_name, if_basename(ifr.ifr_name));
   /* Segfaults if using strncpy(IFNAMSIZ) why oh why?? */
   ifp->int_next = ifnet;
   ifnet = ifp;
@@ -915,14 +943,15 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
 	{
 	  fprintf(stderr, "Could not initialize socket... exiting!\n\n");
 	  olsr_syslog(OLSR_LOG_ERR, "Could not initialize socket... exiting!\n\n");
-	  exit_value = EXIT_FAILURE;
+	  olsr_cnf->exit_value = EXIT_FAILURE;
 	  kill(getpid(), SIGINT);
 	}
+
     }
   else
     {
       /* IP version 6 */
-      memcpy(&ifp->ip_addr, &ifp->int6_addr.sin6_addr, ipsize);
+      memcpy(&ifp->ip_addr, &ifp->int6_addr.sin6_addr, olsr_cnf->ipsize);
 
       
       /*
@@ -939,12 +968,14 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
 	{
 	  fprintf(stderr, "Could not initialize socket... exiting!\n\n");
 	  olsr_syslog(OLSR_LOG_ERR, "Could not initialize socket... exiting!\n\n");
-	  exit_value = EXIT_FAILURE;
+	  olsr_cnf->exit_value = EXIT_FAILURE;
 	  kill(getpid(), SIGINT);
 	}
       
     }
-  
+
+  set_buffer_timer(ifp);
+
   /* Register socket */
   add_olsr_socket(ifp->olsr_socket, &olsr_input);
   
@@ -971,12 +1002,12 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
   /*
    * Set main address if this is the only interface
    */
-  memset(&null_addr, 0, ipsize);
-  if(COMP_IP(&null_addr, &main_addr))
+  memset(&null_addr, 0, olsr_cnf->ipsize);
+  if(COMP_IP(&null_addr, &olsr_cnf->main_addr))
     {
-      COPY_IP(&main_addr, &ifp->ip_addr);
-      OLSR_PRINTF(1, "New main address: %s\n", olsr_ip_to_string(&main_addr))
-      olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", olsr_ip_to_string(&main_addr));
+      COPY_IP(&olsr_cnf->main_addr, &ifp->ip_addr);
+      OLSR_PRINTF(1, "New main address: %s\n", olsr_ip_to_string(&olsr_cnf->main_addr));
+      olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", olsr_ip_to_string(&olsr_cnf->main_addr));
     }
   
   /*
@@ -1024,12 +1055,13 @@ chk_if_up(struct olsr_if *iface, int debuglvl)
 
   /* Recalculate max jitter */
 
-  if((max_jitter == 0) || ((iface->cnf->hello_params.emission_interval / 4) < max_jitter))
-    max_jitter = iface->cnf->hello_params.emission_interval / 4;
+  if((olsr_cnf->max_jitter == 0) || 
+     ((iface->cnf->hello_params.emission_interval / 4) < olsr_cnf->max_jitter))
+    olsr_cnf->max_jitter = iface->cnf->hello_params.emission_interval / 4;
 
   /* Recalculate max topology hold time */
-  if(max_tc_vtime < iface->cnf->tc_params.emission_interval)
-    max_tc_vtime = iface->cnf->tc_params.emission_interval;
+  if(olsr_cnf->max_tc_vtime < iface->cnf->tc_params.emission_interval)
+    olsr_cnf->max_tc_vtime = iface->cnf->tc_params.emission_interval;
 
   ifp->hello_etime = iface->cnf->hello_params.emission_interval;
   ifp->valtimes.hello = double_to_me(iface->cnf->hello_params.validity_time);
